#ifndef _BBSKEYS_INCLUDED
#define _BBSKEYS_INCLUDED

/*
  this keys appears in the menus, you may change it, but let it be
  uppercases
*/
#define HELP_KEY	'?'
#define DOC_KEY		'A'
#define SYS_KEY		'I'
#define DIR_KEY		'D'
#define CD_KEY		'C'
#define PWD_KEY		'W'
#define SHOW_KEY	'S'
#define DOWNLOAD_KEY	'G'
#define UPLOAD_KEY	'P'
#define QUIT_KEY	'Q'
#define ENV_KEY		'E'
#define LANG_KEY	'L'
#define PROT_KEY	'P'
#define AUTOZ_KEY	'A'
#define FULLIST_KEY	'F'
#define TERM_KEY	'T'
#define PASSWD_KEY	'N'
#define BBS_KEY		'B'
#define TALK_KEY	'T'
/*
  must be lowercase of QUIT_KEY
*/
#define QUIT_L_KEY	'q'

/*
  this defs should only be changed if necessary
*/
#define AUTOZDETECTED	'p'	/* must be lowercases */
#define ALLQUANTOR_KEY	'*'
#define NO_KEY		0

#define BELL_KEY	'\a'
#define ESCAPE_KEY	27
#define SPACE_KEY	' '
#define NEWLINE_KEY	'\n'
#define RETURN_KEY	'\r'
#define BACKSPACE_KEY	'\b'
#define DELETE_KEY	127

#endif
