#ifndef _BBS_HPUX_INCLUDED
#define _BBS_HPUX_INCLUDED

#define TGETENTBUF 1024
#define FD_SETPTR_CAST (int *)

#ifdef __GNUC__
extern syslog(int, const char *, ...);
extern openlog(const char *, int, int);
extern closelog(void);
extern int crmode(void);
#endif

#endif  /* _BBS_HPUX_INCLUDED */
