#ifndef _BBS_NEXT_INCLUDED
#define _BBS_NEXT_INCLUDED

#ifndef _POSIX_SOURCE
#error "should be compiled as a POSIX-source (compileroption -posix)"
#endif

#define TGETENTBUF 1024

extern int box (WINDOW *, char, char);
extern int delwin (WINDOW *);
extern int endwin (void);
extern int mvcur (int, int, int, int);
extern int printw (const char *, ...);
extern int touchwin(WINDOW *);
extern int setterm (char *);
extern int waddch (WINDOW *, char);
extern int waddstr (WINDOW *, const char *);
extern int wdelch (WINDOW *);
extern int wclear (WINDOW *);
extern int wclrtobot (WINDOW *);
extern int wclrtoeol (WINDOW *);
extern int werase (WINDOW *);
extern int wgetch (WINDOW *);
extern int wgetstr (WINDOW *, char *);
extern int wmove (WINDOW *, int, int);
extern int wprintw (WINDOW *, const char *, ...);
extern int wrefresh (WINDOW *);
extern int wstandend (WINDOW *);
extern int wstandout (WINDOW *);
extern void bzero(void *, size_t);
extern void bcopy(const void *, void *, size_t);
extern char *index(const char *, int);
extern int tgetent(char *, const char *);

#include </usr/include/ansi/limits.h>
#include <libc.h>

#endif  /* _BBS_NEXT_INCLUDED */
