#ifndef _BBS_LINUX_INCLUDED
#define _BBS_LINUX_INCLUDED

#ifndef _POSIX_SOURCE
#error "should be compiled as a POSIX-source (compileroption -posix)"
#endif

#include <sys/time.h>

#define TGETENTBUF 2048

#ifndef __USE_BSD
extern char *getpass(const char *);
extern long int random(void);
extern void srandom(unsigned int);
#endif

#ifndef  __USE_POSIX2
extern int getopt(int, char * const [], const char *);
#endif

#ifndef  __USE_SVID
extern int putenv(const char *);
#endif

#endif  /* _BBS_LINUX_INCLUDED */
