#ifndef _BBSH_INCLUDED
#define _BBSH_INCLUDED

#include <curses.h>
#include <unistd.h>
#include <stdlib.h>
#include <regex.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <signal.h>
#include <setjmp.h>
#include <syslog.h>
#include <errno.h>
#include <pwd.h>
#include <dirent.h>
#include <termios.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/param.h>
#include <sys/mman.h>

#include "local.h"
#include "bbs_types.h"
#include "bbs_decl.h"

extern volatile SIG_ATOMIC_T signumber, canjump_sigalrm, canjump_sigenv,
       canjump_talkaccept, canjump_sigusr1, canjump_sigusr2;
extern SIGSET_T old_blocksigset;
extern sigmaskstatusenum blocksigsetstatus;
extern sigjmp_buf sigenv, sigalrmenv, talkacceptenv, sigusr1env, sigusr2env;
extern const char *protokollnamen[];
extern const char *sprachennamen[];
extern const char *sprachensuffix[];
extern boolean bbsdaemon;

#endif
